/*
 * Copyright (C) 2021  Christopher J. Howard
 *
 * This file is part of Antkeeper source code.
 *
 * Antkeeper source code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Antkeeper source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Antkeeper source code.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "collision.hpp"
#include "game/component/transform.hpp"

namespace game {
namespace system {

collision::collision(entity::registry& registry):
	updatable(registry)
{
	registry.on_construct<component::collision>().connect<&collision::on_collision_construct>(this);
	registry.on_update<component::collision>().connect<&collision::on_collision_update>(this);
	registry.on_destroy<component::collision>().connect<&collision::on_collision_destroy>(this);
}

void collision::update(double t, double dt)
{
	registry.on_construct<component::collision>().disconnect<&collision::on_collision_construct>(this);
	registry.on_update<component::collision>().disconnect<&collision::on_collision_update>(this);
	registry.on_destroy<component::collision>().disconnect<&collision::on_collision_destroy>(this);
}

void collision::on_collision_construct(entity::registry& registry, entity::id entity_id)
{}

void collision::on_collision_update(entity::registry& registry, entity::id entity_id)
{}

void collision::on_collision_destroy(entity::registry& registry, entity::id entity_id)
{}

} // namespace system
} // namespace game
